package ca.canadapost.cpcdp.tracking;

import ca.canadapost.cpcdp.tracking.generated.messages.Messages;
import ca.canadapost.cpcdp.tracking.generated.track.*;

import com.sun.jersey.api.client.*;
import com.sun.jersey.api.client.config.*;

import javax.xml.bind.JAXBContext;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

import java.io.FileOutputStream;
import org.apache.commons.codec.binary.Base64;

/**
 * Sample code for the GetDeliveryConfirmationCertificate Canada Post service.
 * <p>
 * If the parcel has been delivered, the GetDeliveryConfirmationCertificate  service  
 * returns a signature image captured at delivery of the parcel if available. (The 
 * parcel is identified using a PIN only). 
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 * <p>
 *  Jersey REST client using JAXB for GetDeliveryConfirmationCertificate Request.  <br>
 *  USAGE:<pre>
 *        GetDCCertifcateClient client = new GetDeliveryConfirmationCertificateClient(username, password);
 *        ClientResponse response = client.getDeliveryConfirmationCertificate();
 *        // do whatever with response
 *        client.close();
 *  </pre>
 *
 */
public class GetDeliveryConfirmationCertificateClient {

    private Client aClient;
    private static final String LINK = "https://ct.soa-gw.canadapost.ca/vis/certificate/1371134583769923";
    
    // Main class for local testing only
    public static void main(String[] args) {
    	
    	// Your username and password are imported from the following file
    	// CPCWS_Tracking_Java_Samples/user.properties 
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");				
		
		// Execute getDeliveryConfirmationCertificate Request
        GetDeliveryConfirmationCertificateClient myClient = new GetDeliveryConfirmationCertificateClient(username, password);
        ClientResponse resp = myClient.getDeliveryConfirmationCertificate();
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());

        // Example of using JAXB to parse xml response
        JAXBContext jc;
        try {
        	jc = JAXBContext.newInstance(DeliveryConfirmationCertificate.class, Messages.class);
            Object entity = jc.createUnmarshaller().unmarshal(respIS);
            // Determine whether response data matches DeliveryConfirmationCertificate schema.
            if (entity instanceof DeliveryConfirmationCertificate) {
            	// Decoding base64 certificate to a file
            	DeliveryConfirmationCertificate certifcate = (DeliveryConfirmationCertificate) entity;
                String fileName = certifcate.getFilename();
                String encodedImage = certifcate.getImage();
                System.out.println("Base64 Encoded: " + encodedImage);
            	System.out.println("File name: " + fileName);
                System.out.println("Mime type: " + certifcate.getMimeType());
                FileOutputStream fos = new FileOutputStream(fileName);
                Base64 decoder = new Base64();
                byte[] decodedBytes = decoder.decode(encodedImage);               
                fos.write(decodedBytes);
                fos.close();
                System.out.println("Decoding to " + System.getProperty("user.dir") + System.getProperty("file.separator") + fileName);
            
            } else {
                // Assume Error Schema
                Messages messageData = (Messages) entity;
                for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
                    Messages.Message aMessage = (Messages.Message) iter.next();
                    System.out.println("Error Code: " + aMessage.getCode());
                    System.out.println("Error Msg: " + aMessage.getDescription());
                }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }
        myClient.close();
    }

    public GetDeliveryConfirmationCertificateClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
    }

    public ClientResponse getDeliveryConfirmationCertificate() throws UniformInterfaceException {
        WebResource aWebResource = aClient.resource(LINK);
        return aWebResource.accept("application/vnd.cpc.track+xml").acceptLanguage("en-CA").get(ClientResponse.class);
    }

    public void close() {
        aClient.destroy();
    }
}
